﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using MVVMwithWPF.Models;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;

namespace MVVMwithWPF.ViewModels
{
    internal partial class MainViewModel : ViewModelBase
    {
        [ObservableProperty]
        private ObservableCollection<Member> _members;
        [ObservableProperty]
        private bool _isEditing;
        [ObservableProperty]
        private Member _editMember;

        private bool CanEditMember { get { return EditMember != null; } }

        public MainViewModel()
        {
            Title = "Main View";
            Init();
        }

        private void Init()
        {
            Members = new ObservableCollection<Member>();
            {
                new Member()
                {
                    Id = 1,
                    Name = "member001",
                    Phone = "010-1234-5678",
                    RegDate = DateTime.Now,
                    IsUse = true
                };
            };

            PropertyChanged += MainViewModel_PropertyChanged;
        }

        [RelayCommand]
        private void Save()
        {
            IsEditing = false;
            if (EditMember.Id  == 0)
            {
                var id = Members.Any() ? Members.Max(m => m.Id) + 1 : 1;
                EditMember.Id = id;
                EditMember.RegDate = DateTime.Now;

                Members.Add((Member)EditMember.Clone());

                EditMember = null;
            }
            else
            {
                var member = Members.FirstOrDefault(m => m.Id == EditMember.Id);
                if (member == null)
                    return;

                member.Name = EditMember.Name;
                member.Phone = EditMember.Phone;
                member.IsUse = EditMember.IsUse;
            }
        }

        [RelayCommand]
        private void Cancle()
        {
            IsEditing = false;
            EditMember = null;
        }

        [RelayCommand(CanExecute = nameof(CanEditMember))]
        private void Delete()
        {
            if (EditMember == null)
                return;

            var result = MessageBox.Show("선택된 아이템을 삭제하시겠습니까?", "확인", MessageBoxButton.YesNo);
            if (result != MessageBoxResult.Yes)
                return;

            IsEditing = false;
            var removeMember = Members.FirstOrDefault(m => m.Id == EditMember.Id);
            if (removeMember != null)
                Members.Remove(removeMember);

            EditMember = null;
        }

        [RelayCommand]
        private void New()
        {
            IsEditing = true;
            EditMember = new Member
            {
                IsUse = true
            };
        }


        [RelayCommand(CanExecute = nameof(CanEditMember))]

        private void Edit()
        {
            IsEditing = true;
        }

        [RelayCommand]
        private void SelectionChanged(object param)
        {
            var args = param as SelectionChangedEventArgs;
            if (args == null)
                return;

            if (args.AddedItems.Count < 1)
            {
                EditMember = null;
                return;
            }

            if (args.AddedItems[0] is Member member)
                EditMember = (Member)member.Clone();
        }

        private void MainViewModel_PropertyChanged(object? sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            switch(e.PropertyName)
            {
                case nameof(EditMember):
                    EditCommand.NotifyCanExecuteChanged();
                    DeleteCommand.NotifyCanExecuteChanged();
                    break;
            }
        }
    }
}
